#ifndef UML_ASSOCIATIONEND_H
#define UML_ASSOCIATIONEND_H

#include <umlmodelelement.h>

class UMLClassifier;
class UMLAttribute;
class UMLAssociation;

/**
 * UML 1.4 section 2.5.2.5
 *
 * An association end is one of the endpoints of an
 * association. An association is somewhat similar to a
 * structural feature except that facilitates the
 * association that connects them.
 *
 * The participant is used to logically attach the
 * association end to a classifier. The specification
 * determine the minimum interface that must be realized
 * by the participant (is that right?) to support the
 * association.
 *
 * Renamed specification to spec because of inheritance
 * collision from UMLModelElement.
 *
 * Removed attribute visibility because of reimplementation
 * from UMLModelElement.
 */
class UMLAssociationEnd : public UMLModelElement
{
 public:
  UMLAssociationEnd();
  virtual ~UMLAssociationEnd();

  utl::single_attribute<UMLBoolean> isNavigable;
  utl::single_attribute<UMLOrderingKind> ordering;
  utl::single_attribute<UMLAggregationKind> aggregation;
  utl::single_attribute<UMLScopeKind> targetScope;
  utl::single_attribute<UMLMultiplicity> multiplicity;
  utl::single_attribute<UMLChangeabilityKind> changeability;

  utl::single_relation<UMLClassifier *> participant;
  utl::unordered_relation<UMLClassifier *> spec;
  utl::ordered_relation<UMLAttribute *> qualifier;
  utl::single_relation<UMLAssociation *> association;
};

#endif
