#ifndef UML_GENERALIZABLEELEMENT_H
#define UML_GENERALIZABLEELEMENT_H

#include <umlmodelelement.h>
#include <umlgeneralization.h>

/**
 * UML 1.4 section 2.5.2.23
 *
 * The generalizable element class is primarily an interface
 * that maintains information on the inheritance information
 * of a model element. This includes the set of generalization
 * elements associated with it.
 *
 * We need to be careful when deleting generalizations and
 * specializations because these relationships are two way.
 * we need a non-destructive way to remove the relationship
 * from the other endpoint - hence the introduction of the
 * non-standard clear method.
 *
 * Deviation from the UML IDL. They seem to use this bizarre
 * parent specialization link instead of just a set of
 * generalizations to children. For the time being, this
 * seems considerably more simple.
 */
class UMLGeneralizableElement : public UMLModelElement
{
 public:
  UMLGeneralizableElement();
  virtual ~UMLGeneralizableElement();

  utl::single_attribute<UMLBoolean> isRoot;
  utl::single_attribute<UMLBoolean> isLeaf;
  utl::single_attribute<UMLBoolean> isAbstract;

  utl::unordered_attribute<UMLGeneralization *> generalization;
  utl::unordered_attribute<UMLGeneralization *> specialization;
};

#endif
