#ifndef UML_MODELELEMENT_H
#define UML_MODELELEMENT_H

#include <umlelement.h>
#include <umldatatypes.h>

class UMLNamespace;
class UMLConstraint;
class UMLFlow;
class UMLDependency;
class UMLTemplateParameter;
class UMLPresentationElement;
class UMLComment;

/**
 * UML 1.4 section 2.5.2.27
 *
 * The model element is the base class of \e most elements
 * in UML. It provies base services and attributes such as
 * naming and constraints to descendant classes.
 *
 * Note that the association with namespace pushes the
 * attributes of element ownership into this class. This means
 * that the model elements capabilities are extended to include
 * namespace resolution and visibility.
 *
 * Also included in the model element is the containment of
 * all associated source and target flows. When source or
 * target flows are removed from the model element, the flows
 * are deleted. Note that when one a flow is removed, it must
 * also be removed from the other endpoint. That activity is
 * actually managed by the flow itself by calling back to the
 * clear methods.
 *
 * Renamed the namespace attribute to ns because of
 * keyword conflict.
 *
 */
class UMLModelElement : public UMLElement
{
 public:
  UMLModelElement();
  virtual ~UMLModelElement();

  utl::single_attribute<UMLName> name;
  utl::single_attribute<UMLVisibilityKind> visibility;
  utl::single_attribute<UMLBoolean> specification;

  utl::single_relation<UMLNamespace *> ns;
  utl::unordered_relation<UMLConstraint *> constraint;
  utl::unordered_relation<UMLFlow *> targetFlow;
  utl::unordered_relation<UMLFlow *> sourceFlow;
  utl::unordered_relation<UMLDependency *> clientDependency;
  utl::unordered_relation<UMLDependency *> supplierDependency;
  utl::ordered_relation<UMLTemplateParameter *> templateParameter;
  utl::unordered_relation<UMLPresentationElement *> presentation;
  utl::unordered_relation<UMLComment *> comment;
};


#endif
