#ifndef UTL_CONNECTOR_H
#define UTL_CONNECTOR_H

namespace utl
{
  class relation;

  /**
   * An endpoint is an element and a relation. The relation
   * member is a pointer, but it is a pointer to an attribute
   * of the element.
   */
  template<class Type>
  struct endpoint
  {
    typedef Type type;

    endpoint() :
      _elem(),
      _rel(0)
    {}

    endpoint(const endpoint<Type> &ep) :
      _elem(ep._elem),
      _rel(ep._rel)
    {}

    endpoint(const type &e, relation *r = 0) :
      _elem(e),
      _rel(r)
    {}

    type _elem;
    relation *_rel;
  };

  /**
   * The connect method takes four parameters: two elements
   * and two relationships. It connects element1 to relation2
   * and relation2 to element1.
   *
   * Note that if either relation is NULL, then the assocation
   * is not made to that relation.
   */
  template<class Element1,
	   class Element2>
  void connect(Element1 e1, relation *r1,
	       Element2 e2, relation *r2)
  {
    endpoint<Element1> ep1(e1, r1);
    endpoint<Element2> ep2(e2, r2);
    if(r1) r1->connect(ep2);
    if(r2) r2->connect(ep1);
  }


  /**
   * The disconnect method takes four parameters: two elements
   * and two relationships. It disconnects element1 from relation2
   * and relation2 from element1.
   */
  template<class Element1,
	   class Element2>
  void disconnect(Element1 e1, relation *r1,
		  Element2 e2, relation *r2)
  {
    endpoint<Element1> ep1(e1, r1);
    endpoint<Element2> ep2(e2, r2);
    r1->disconnect(ep2);
    r2->disconnect(ep1);
  }
};

#endif
